////////////////////
// Initialization //
////////////////////

// Do various startup things
init()
{
	// Kill any global threads that are running
	level notify("cod2_killthreads");

	SetupVariables();

	// Init stuff	
	cod2\_mortars::init();
	
	DoPrecaching();
	LoadEffects();

	Setup();
}

// Read cvars and setup variables
SetupVariables()
{
	////////////////////////////
	// Setup global variables //
	////////////////////////////
	
	level.cod2_gametype = getcvar("g_gametype");		// Save gametype in case we are pretending to be another gametype

	switch(level.cod2_gametype)
	{
		case "dm":
			level.cod2_teamplay = false;
			break;

		case "lms":
			level.cod2_teamplay = false;
			break;

		case "ihtf":
			level.cod2_teamplay = false;
			break;

		default:
			level.cod2_teamplay = true;
			break;
	}

	// defaults if not defined in level script
	if(!isDefined(game["allies"]))	game["allies"] = "american";
	if(!isDefined(game["axis"]))		game["axis"] = "german";

	// Number of available punishments
	level.cod2_punishments = 3;

	if(!isdefined(game["roundsplayed"]))
		level.cod2_roundbased = false;
	else
		level.cod2_roundbased = true;

	// Create player array
	level.cod2_allplayers = getentarray("player", "classname");

	// Set up number of voices
	level.cod2_voices["german"] = 3;
	level.cod2_voices["american"] = 7;
	level.cod2_voices["russian"] = 6;
	level.cod2_voices["british"] = 6;

	if(isdefined(game["german_soldiertype"]) && (game["german_soldiertype"] == "winterlight" || game["german_soldiertype"] == "winterdark") )
		level.cod2_wintermap = true;
	else
		level.cod2_wintermap = false;

	////////////////
	// Read cvars //
	////////////////

	// Debug
	level.cod2_debug 			= cod2\_util::cvardef("cod2_debug", 0, 0, 1, "int");	

     	// pain & death sounds
	level.cod2_painsound		= cod2\_util::cvardef("cod2_painsound", 1, 0, 1, "int");
	level.cod2_deathsound		= cod2\_util::cvardef("cod2_deathsound", 1, 0, 1, "int");

	// Use deathicons, nadeicons, weapon drop etc...
	level.cod2_allowcrosshair	= cod2\_util::cvardef("cod2_allow_crosshair", 1, 0, 2, "int");
	level.cod2_allowcrosshairnames	= cod2\_util::cvardef("cod2_allow_crosshair_names", 1, 0, 2, "int");
	level.cod2_allowcrosshaircolor	= cod2\_util::cvardef("cod2_allow_crosshair_color", 1, 0, 2, "int");
	
	// Unknown Soldiers handling
	level.cod2_unknownreflect 	= cod2\_util::cvardef("cod2_unknown_reflect",1,0,1,"int");
	level.cod2_unknownmethod 	= cod2\_util::cvardef("cod2_unknown_method",0,0,2,"int");
	level.cod2_unknownrenamemsg 	= cod2\_util::cvardef("cod2_unknown_rename_msg","Unknown Soldier is not a valid name! You have been renamed by the server.","","","string");

	// Override gravity?
	level.cod2_gravity 		= cod2\_util::cvardef("cod2_gravity",100,0,9999,"int");
	// Override speed?
	level.cod2_speed	 		= cod2\_util::cvardef("cod2_speed",100,0,9999,"int");

	// Crush models
	level.cod2_crushmodels = [];
	level.cod2_crushmodels[level.cod2_crushmodels.size] = cod2\_util::cvardef("cod2_crush_model0","furniture_piano_d","","","string");
	for(i=1;i<100;i++)
	{
		crushmodel = cod2\_util::cvardef("cod2_crush_model" + i,"none","","","string");
		if(crushmodel == "none") break;
		level.cod2_crushmodels[level.cod2_crushmodels.size] = crushmodel;
	}

	// Stop client exploits
	level.cod2_stopclientexploits	= cod2\_util::cvardef("cod2_stop_client_exploits",1,0,1,"int");

	// Force quick fading compass dots
	level.cod2_quickfadecompassdots= cod2\_util::cvardef("cod2_quickfade_compass_dots",0,0,1,"int");
}

DoPrecaching()
{
	if(!isdefined(game["gamestarted"]))
	{
		if(level.cod2_debug)
		{
			precacheModel("xmodel/vehicle_stuka_flying");
		}
	}
}

LoadEffects()
{
	// Used to blow up players. Using same effect as sd.gsc (should save resources)
	level._effect["bombexplosion"] = loadfx("fx/props/barrelexp.efx");
}

Setup()
{
	// Find map limits
	cod2\_util::FindMapDimensions();

	// Find play area by checking all the spawnpoints
	cod2\_util::FindPlayArea();

	// Override speed and/or gravity?
	if(level.cod2_gravity != 100) setcvar("g_gravity", 8 * level.cod2_gravity);
	if(level.cod2_speed != 100) setcvar("g_speed", int(1.9 * level.cod2_speed) );

	// Warm up round
//	WarmupRound();
}


///////////////
// Functions //
///////////////

EndMap()
{
	if(level.cod2_disable)
		return;

	setcvar("g_gametype",level.cod2_gametype);		// Restore gametype in case we are pretending

//	cod2\_mapvote::Initialise();
}
